﻿using System;
using System.Collections;
using System.Collections.Generic;
using System.Threading;
using RocketAd;
using RocketGM;
using RocketLog;
using UnityEngine;

namespace RocketRevenueTracker
{
    public class RocketRevenueTrackerController : MonoBehaviour
    {
        [SerializeField]
        private RocketRevenueTracker[] _revenueTrackers = new RocketRevenueTracker[]
        {
            new RocketRevenueTracker(0.03f, "_03", true),
            new RocketRevenueTracker(0.05f, "_05", true),
            new RocketRevenueTracker(0.10f, "_10", true),
            new RocketRevenueTracker(0.20f, "_20", true),
            new RocketRevenueTracker(0.50f, "_50", true),
            new RocketRevenueTracker(1.00f, "_100", true)
        };

        [SerializeField]
        private int _revenueFlushIntervalInSeconds;

        private List<double> _trackedRevenues;
        private WaitForSeconds _waitForSeconds;
        
#if ROC_GM_APPSFLYER
        [SerializeField]
        private bool _sendAppsflyer = true;
#endif
#if ROC_FACEBOOK
        [SerializeField]
        private bool _sendFacebook = true;
#endif
#if ROC_FIREBASE
[SerializeField]
        private bool _sendFirebase = true;
#endif

        private static int _unityThreadId = -1;

        private RocLog Log = new RocLog(nameof(RocketRevenueTrackerController), DebugLevels.Debug);

        private void Start()
        {
            DontDestroyOnLoad(this);

            _trackedRevenues = new List<double>();
            _waitForSeconds = new WaitForSeconds(_revenueFlushIntervalInSeconds);
            
            _unityThreadId = Thread.CurrentThread.ManagedThreadId;
            Log.Debug($"Initializing RocGMRevenueTracker.\nThread ID: {_unityThreadId}");
            
            Log.Debug($"{_revenueTrackers.Length} tracker found.");

#if ROC_GM_APPSFLYER
            RocketRevenueTracker.SendAppsflyer = _sendAppsflyer;
#endif
#if ROC_FACEBOOK
            RocketRevenueTracker.SendFacebook = _sendFacebook;
#endif
#if ROC_FIREBASE
            RocketRevenueTracker.SendFirebase = _sendFirebase;
#endif
            
            RocGm.DoAfterInitialized(() =>
            {
                foreach (var revenueTracker in _revenueTrackers)
                {
                    revenueTracker.TrySendRevenue(revenueTracker.RevenueAmount);
                }
            });

            StartCoroutine(RevenueFlushEnumerator());
        }

        private void OnEnable()
        {
            RocAd.ImpressionTracked += OnImpressionTracked;
        }

        private void OnDisable()
        {
            RocAd.ImpressionTracked -= OnImpressionTracked;
        }

        private void OnApplicationFocus(bool hasFocus)
        {
            if (!hasFocus)
            {
                FlushCachedRevenues();
            }
        }

        private IEnumerator RevenueFlushEnumerator()
        {
            while (gameObject.activeSelf)
            {
                yield return _waitForSeconds;
                FlushCachedRevenues();
            }
        }

        private void FlushCachedRevenues()
        {
            Log.Debug($"Flushing cached revenues to the related SDKs, sending {_trackedRevenues.Count} revenues.");
            
            foreach (var revenueTracker in _revenueTrackers)
            {
                foreach (var trackedRevenue in _trackedRevenues)
                {
                    revenueTracker.RevenueAmount += (float)trackedRevenue;
                    revenueTracker.TrySendRevenue(revenueTracker.RevenueAmount);
                }
            }
            
            // Clear the tracked revenue list.
            _trackedRevenues.Clear();
        }

        private void OnImpressionTracked(RocImpressionData rocImpressionData)
        {
            Log.Debug($"Tracked/cached revenue for revenue tracker, the revenue was: {rocImpressionData.Revenue}");
            _trackedRevenues.Add(rocImpressionData.Revenue);
        }
    }
}